import { useContext, useEffect } from 'react';
import { KeyboardContext } from './KeyboardProvider';
const modifiers = ['ctrl', 'meta', 'shift', 'alt'];
function parseHandlers(handlers) {
    return Object.keys(handlers).map(key => {
        const output = {
            name: key,
            handler: handlers[key],
            key: '',
        };
        const tokens = key.split('+');
        tokens.forEach((token, index) => {
            if (index === tokens.length - 1) {
                output.key = token;
                return;
            }
            const lowerCaseToken = token.toLowerCase();
            modifiers.forEach((mod) => {
                if (lowerCaseToken === mod) {
                    output[mod] = true;
                }
            });
        });
        return output;
    });
}
export default function useKeyboard({ id, handlers, global, autoFocus, }) {
    const { focusedElementID, setFocusedElementID } = useContext(KeyboardContext);
    const hasFocus = global || focusedElementID === id;
    useEffect(() => {
        if (!autoFocus) {
            return;
        }
        focus();
    }, [autoFocus]);
    useEffect(() => {
        if (!hasFocus) {
            return;
        }
        const parsed = parseHandlers(handlers);
        function onKeyDown(event) {
            const matches = parsed.filter(x => x.key === event.code);
            if (!matches.length) {
                return;
            }
            for (const match of matches) {
                for (const mod of modifiers) {
                    if (match[mod] && event[`${mod}Key`]) {
                        match.handler(event);
                    }
                }
                match.handler(event);
            }
        }
        document.addEventListener('keydown', onKeyDown);
        return () => document.removeEventListener('keydown', onKeyDown);
    }, [handlers, hasFocus]);
    function focus() {
        setFocusedElementID(id);
    }
    return { focus, hasFocus };
}
